import os
import sys
from collections.abc import Mapping
from inspect import ismodule, getfile


class ReadOnlyDict(Mapping):

    def __init__(self, dict_):
        self._data = dict_

    def __getitem__(self, key):
        return self._data[key]

    def __iter__(self):
        return iter(self._data)

    def __len__(self):
        return len(self._data)


class Settings:

    __slots__ = ('_data',)

    PYTEST_SCRIPT_NAME = 'py.test'

    def __init__(self, primary_settings, custom_settings=None, test_settings=None):
        self._data = {}
        self._load_settings(primary_settings)
        if custom_settings:
            self._load_settings(custom_settings)
        if test_settings and self._is_test_running():
            self._load_settings(test_settings)

    def __getattr__(self, name):
        if name in self._data:
            return self._data[name]
        else:
            raise AttributeError("{} attribute doesn't exist".format(name))

    def _is_test_running(self):
        if self.PYTEST_SCRIPT_NAME == os.path.basename(sys.argv[0]):
            return True
        else:
            return False

    def _load_settings_from_file(self, file_path):
        data = {**self._data}
        if os.path.isfile(file_path):
            exec(open(file_path).read(), {}, data)
            self._data = ReadOnlyDict(data)
        else:
            raise FileNotFoundError("File {} doesn't exist".format(file_path))

    def _load_settings(self, settings):
        if ismodule(settings):
            file_path = getfile(settings)
            self._load_settings_from_file(file_path)

        elif isinstance(settings, str):
            if settings in os.environ:
                file_path = os.environ.get(settings)
            else:
                file_path = settings
            self._load_settings_from_file(file_path)

        else:
            raise TypeError('Settings must be module, file path or env variable')



import os
import sys
from inspect import getfile
from importlib.util import spec_from_file_location, module_from_spec

import pytest

from my_settings.settings import Settings


class TestSettings:

    PRIMARY_SETTINGS_VALUE = {'1': 4}
    CUSTOM_SETTINGS_VALUE = (1, 2, 3)
    TEST_SETTINGS_VALUE = {5, 6, 7}
    ENV_SETTINGS = 'TEST_MODULE_PATH'
    PRIMARY_MODULE_DATA = 'TEST_SETTING = {}'.format(PRIMARY_SETTINGS_VALUE)
    CUSTOM_MODULE_DATA = 'TEST_SETTING = {}'.format(CUSTOM_SETTINGS_VALUE)
    TEST_MODULE_DATA = 'TEST_SETTING = {}'.format(TEST_SETTINGS_VALUE)

    def create_module_with_data(self, tmpdir, name, data):
        test_module_file = tmpdir.join(name)
        test_module_file.write(data)
        name, ext = os.path.splitext(name)
        test_module_spec = spec_from_file_location(name, test_module_file.strpath)
        test_module = module_from_spec(test_module_spec)
        return test_module

    def test_load_primary_settings_as_module(self, tmpdir):
        test_module = self.create_module_with_data(tmpdir, 'test_module.py', self.PRIMARY_MODULE_DATA)
        settings = Settings(test_module)

        assert settings.TEST_SETTING == self.PRIMARY_SETTINGS_VALUE

    def test_load_primary_settings_as_env_var(self, tmpdir):
        test_module = self.create_module_with_data(tmpdir, 'test_module.py', self.PRIMARY_MODULE_DATA)
        os.environ[self.ENV_SETTINGS] = getfile(test_module)
        settings = Settings(self.ENV_SETTINGS)

        assert settings.TEST_SETTING == self.PRIMARY_SETTINGS_VALUE

    def test_load_primary_settings_as_file_path(self, tmpdir):
        test_module = self.create_module_with_data(tmpdir, 'test_module.py', self.PRIMARY_MODULE_DATA)
        settings = Settings(getfile(test_module))

        assert settings.TEST_SETTING == self.PRIMARY_SETTINGS_VALUE

    def test_load_custom_settings(self, tmpdir):
        primary_settings = self.create_module_with_data(tmpdir, 'primary_module.py', self.PRIMARY_MODULE_DATA)
        custom_settings = self.create_module_with_data(tmpdir, 'custom_module.py', self.CUSTOM_MODULE_DATA)
        settings = Settings(primary_settings, custom_settings)

        assert settings.TEST_SETTING == self.CUSTOM_SETTINGS_VALUE

    def test_load_test_settings(self, tmpdir):
        primary_settings = self.create_module_with_data(tmpdir, 'primary_module.py', self.PRIMARY_MODULE_DATA)
        custom_settings = self.create_module_with_data(tmpdir, 'custom_module.py', self.CUSTOM_MODULE_DATA)
        test_settings = self.create_module_with_data(tmpdir, 'test_module.py', self.TEST_MODULE_DATA)
        settings = Settings(primary_settings, custom_settings, test_settings)

        assert settings.TEST_SETTING == self.TEST_SETTINGS_VALUE

    def test_load_test_setting_without_custom_settings(self, tmpdir):
        primary_settings = self.create_module_with_data(tmpdir, 'primary_module.py', self.PRIMARY_MODULE_DATA)
        test_settings = self.create_module_with_data(tmpdir, 'test_module.py', self.TEST_MODULE_DATA)
        settings = Settings(primary_settings, test_settings=test_settings)

        assert settings.TEST_SETTING == self.TEST_SETTINGS_VALUE

    def test_dont_load_test_settings(self, tmpdir, monkeypatch):
        primary_settings = self.create_module_with_data(tmpdir, 'primary_module.py', self.PRIMARY_MODULE_DATA)
        test_settings = self.create_module_with_data(tmpdir, 'test_module.py', self.TEST_MODULE_DATA)

        monkeypatch.setattr(sys, 'argv', [''])
        settings = Settings(primary_settings, test_settings=test_settings)

        assert settings.TEST_SETTING == self.PRIMARY_SETTINGS_VALUE

    def test_cant_change_settings(self, tmpdir):
        primary_settings = self.create_module_with_data(tmpdir, 'primary_module.py', self.PRIMARY_MODULE_DATA)

        settings = Settings(primary_settings)
        with pytest.raises(AttributeError):
            settings.TEST_SETTING = 1



from setuptools import setup, find_packages

from my_settings import __version__


INSTALL_REQUIRES = ()
EXTRAS_REQUIRE = {'dev': ('pytest==3.2.0', 'pdbpp==0.9.1')}
LONG_DESCRIPTION = '''
my_settings can load your settings from python module, 
env variable and file path which has to lead to python module.
You can have three types of settings files:
1) primary(usually plays role of base settings file which is included in a project) 
2) custom(your custom settings file for certain deployment case)
3) test(your settings file for tests)

all settings files overlap each other in this sequence: test -> custom -> primary
'''



setup(
    name='my_settings',
    author='Greg Eremeev',
    author_email='gregory.eremeev@gmail.com',
    version=__version__,
    description='Convenient and simple settings for any project',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/plain',
    license='MIT',
    packages=find_packages(),
    url='https://github.com/GregEremeev/my_settings',
    zip_safe=False,
    python_requires='>=3.3',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    platforms='any',
    install_requires=INSTALL_REQUIRES,
    extras_require=EXTRAS_REQUIRE,
    include_package_data=True,
)
